@@ -432,16 +432,495 @@ public abstract class HttpWagonTestCase
 
         if ( supportProxyPreemptiveAuthentication() )
         {
-            assertEquals( 200, handler.securityHandlerRequestReponses.get( 0 ).responseCode );
+            assertEquals( 200, handler.handlerRequestResponses.get( 0 ).responseCode );
         }
         else
         {
-            assertEquals( 407, handler.securityHandlerRequestReponses.get( 0 ).responseCode );
-            assertEquals( 200, handler.securityHandlerRequestReponses.get( 1 ).responseCode );
+            assertEquals( 407, handler.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( 200, handler.handlerRequestResponses.get( 1 ).responseCode );
         }
 
     }
 
+    public void testRedirectGetToStream()
+        throws Exception
+    {
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+
+        Server server = new Server( 0 );
+        TestHeaderHandler handler = new TestHeaderHandler();
+
+        server.setHandler( handler );
+        addConnectors( server );
+        server.start();
+
+        Server redirectServer = new Server( 0 );
+
+        addConnectors( redirectServer );
+
+        String protocol = getProtocol();
+
+        // protocol is wagon protocol but in fact dav is http(s)
+        if ( protocol.equals( "dav" ) )
+        {
+            protocol = "http";
+        }
+
+        if ( protocol.equals( "davs" ) )
+        {
+            protocol = "https";
+        }
+
+        String redirectUrl = protocol + "://localhost:" + server.getConnectors()[0].getLocalPort();
+
+        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, null );
+
+        redirectServer.setHandler( redirectHandler );
+
+        redirectServer.start();
+
+        wagon.connect( new Repository( "id", getRepositoryUrl( redirectServer ) ) );
+
+        File tmpResult = File.createTempFile( "foo", "get" );
+
+        FileOutputStream fileOutputStream = new FileOutputStream( tmpResult );
+
+        try
+        {
+            wagon.getToStream( "resource", fileOutputStream );
+            fileOutputStream.flush();
+            fileOutputStream.close();
+            String found = FileUtils.fileRead( tmpResult );
+            assertEquals( "found:'" + found + "'", "Hello, World!", found );
+
+            assertEquals( 1, handler.handlerRequestResponses.size() );
+            assertEquals( 200, handler.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( 1, redirectHandler.handlerRequestResponses.size() );
+            assertEquals( 302, redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+        }
+        finally
+        {
+            wagon.disconnect();
+
+            server.stop();
+
+            tmpResult.delete();
+        }
+    }
+
+    public void testRedirectGet()
+        throws Exception
+    {
+        StreamingWagon wagon = (StreamingWagon) getWagon();
+
+        Server server = new Server( 0 );
+        TestHeaderHandler handler = new TestHeaderHandler();
+
+        server.setHandler( handler );
+        addConnectors( server );
+        server.start();
+
+        Server redirectServer = new Server( 0 );
+
+        addConnectors( redirectServer );
+
+        String protocol = getProtocol();
+
+        // protocol is wagon protocol but in fact dav is http(s)
+        if ( protocol.equals( "dav" ) )
+        {
+            protocol = "http";
+        }
+
+        if ( protocol.equals( "davs" ) )
+        {
+            protocol = "https";
+        }
+
+        String redirectUrl = protocol + "://localhost:" + server.getConnectors()[0].getLocalPort();
+
+        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, null );
+
+        redirectServer.setHandler( redirectHandler );
+
+        redirectServer.start();
+
+        wagon.connect( new Repository( "id", getRepositoryUrl( redirectServer ) ) );
+
+        File tmpResult = File.createTempFile( "foo", "get" );
+
+        try
+        {
+            wagon.get( "resource", tmpResult );
+            String found = FileUtils.fileRead( tmpResult );
+            assertEquals( "found:'" + found + "'", "Hello, World!", found );
+
+            assertEquals( 1, handler.handlerRequestResponses.size() );
+            assertEquals( 200, handler.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( 1, redirectHandler.handlerRequestResponses.size() );
+            assertEquals( 302, redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+        }
+        finally
+        {
+            wagon.disconnect();
+
+            server.stop();
+
+            tmpResult.delete();
+        }
+    }
+
+
+    public void testRedirectPutFromStreamWithFullUrl()
+        throws Exception
+    {
+        Server realServer = new Server( 0 );
+
+        addConnectors( realServer );
+
+        File repositoryDirectory = getRepositoryDirectory();
+        FileUtils.deleteDirectory( repositoryDirectory );
+        repositoryDirectory.mkdirs();
+
+        PutHandler putHandler = new PutHandler( repositoryDirectory );
+
+        realServer.setHandler( putHandler );
+
+        realServer.start();
+
+        Server redirectServer = new Server( 0 );
+
+        addConnectors( redirectServer );
+
+        String protocol = getProtocol();
+
+        // protocol is wagon protocol but in fact dav is http(s)
+        if ( protocol.equals( "dav" ) )
+        {
+            protocol = "http";
+        }
+
+        if ( protocol.equals( "davs" ) )
+        {
+            protocol = "https";
+        }
+
+        String redirectUrl = protocol + "://localhost:" + realServer.getConnectors()[0].getLocalPort();
+
+        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, repositoryDirectory );
+
+        redirectServer.setHandler( redirectHandler );
+
+        redirectServer.start();
+
+        try
+        {
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+            Repository repository = new Repository( "foo", getRepositoryUrl( redirectServer ) );
+            wagon.connect( repository );
+
+            File sourceFile = new File( repositoryDirectory, "test-secured-put-resource" );
+            sourceFile.delete();
+            assertFalse( sourceFile.exists() );
+
+            File tempFile = File.createTempFile( "wagon", "tmp" );
+            tempFile.deleteOnExit();
+            String content = "put top secret";
+            FileUtils.fileWrite( tempFile.getAbsolutePath(), content );
+
+            FileInputStream fileInputStream = new FileInputStream( tempFile );
+            try
+            {
+                wagon.putFromStream( fileInputStream, "test-secured-put-resource", content.length(), -1 );
+            }
+            finally
+            {
+                fileInputStream.close();
+                tempFile.delete();
+
+            }
+
+            assertEquals( content, FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+
+            checkRequestResponseForRedirectPutFromStreamWithFullUrl( putHandler, redirectHandler );
+        }
+        finally
+        {
+            realServer.stop();
+            redirectServer.stop();
+        }
+    }
+
+    protected void checkRequestResponseForRedirectPutFromStreamWithFullUrl( PutHandler putHandler,
+                                                                            RedirectHandler redirectHandler )
+    {
+        assertEquals( "found:" + putHandler.handlerRequestResponses, 1, putHandler.handlerRequestResponses.size() );
+        assertEquals( "found:" + putHandler.handlerRequestResponses, 201,
+                      putHandler.handlerRequestResponses.get( 0 ).responseCode );
+        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 1,
+                      redirectHandler.handlerRequestResponses.size() );
+        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 302,
+                      redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+    }
+
+    public void testRedirectPutFromStreamRelativeUrl()
+        throws Exception
+    {
+        Server realServer = new Server( 0 );
+        addConnectors( realServer );
+        File repositoryDirectory = getRepositoryDirectory();
+        FileUtils.deleteDirectory( repositoryDirectory );
+        repositoryDirectory.mkdirs();
+
+        PutHandler putHandler = new PutHandler( repositoryDirectory );
+
+        realServer.setHandler( putHandler );
+
+        realServer.start();
+
+        Server redirectServer = new Server( 0 );
+
+        addConnectors( redirectServer );
+
+        RedirectHandler redirectHandler =
+            new RedirectHandler( "Found", 303, "/redirectRequest/foo", repositoryDirectory );
+
+        redirectServer.setHandler( redirectHandler );
+
+        redirectServer.start();
+
+        try
+        {
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+            Repository repository = new Repository( "foo", getRepositoryUrl( redirectServer ) );
+            wagon.connect( repository );
+
+            File sourceFile = new File( repositoryDirectory, "/redirectRequest/foo/test-secured-put-resource" );
+            sourceFile.delete();
+            assertFalse( sourceFile.exists() );
+
+            File tempFile = File.createTempFile( "wagon", "tmp" );
+            tempFile.deleteOnExit();
+            String content = "put top secret";
+            FileUtils.fileWrite( tempFile.getAbsolutePath(), content );
+
+            FileInputStream fileInputStream = new FileInputStream( tempFile );
+            try
+            {
+                wagon.putFromStream( fileInputStream, "test-secured-put-resource", content.length(), -1 );
+            }
+            finally
+            {
+                fileInputStream.close();
+                tempFile.delete();
+
+            }
+
+            assertEquals( content, FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+
+            checkRequestResponseForRedirectPutFromStreamWithRelativeUrl( putHandler, redirectHandler );
+
+        }
+        finally
+        {
+            realServer.stop();
+            redirectServer.stop();
+        }
+    }
+
+    protected void checkRequestResponseForRedirectPutFromStreamWithRelativeUrl( PutHandler putHandler,
+                                                                                RedirectHandler redirectHandler )
+    {
+        assertEquals( "found:" + putHandler.handlerRequestResponses, 0, putHandler.handlerRequestResponses.size() );
+
+        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 2,
+                      redirectHandler.handlerRequestResponses.size() );
+        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 302,
+                      redirectHandler.handlerRequestResponses.get( 0 ).responseCode );
+        assertEquals( "found:" + redirectHandler.handlerRequestResponses, 201,
+                      redirectHandler.handlerRequestResponses.get( 1 ).responseCode );
+
+    }
+
+    public void testRedirectPutFileWithFullUrl()
+        throws Exception
+    {
+        Server realServer = new Server( 0 );
+
+        addConnectors( realServer );
+
+        File repositoryDirectory = getRepositoryDirectory();
+        FileUtils.deleteDirectory( repositoryDirectory );
+        repositoryDirectory.mkdirs();
+
+        PutHandler putHandler = new PutHandler( repositoryDirectory );
+
+        realServer.setHandler( putHandler );
+
+        realServer.start();
+
+        Server redirectServer = new Server( 0 );
+
+        addConnectors( redirectServer );
+
+        String protocol = getProtocol();
+
+        // protocol is wagon protocol but in fact dav is http(s)
+        if ( protocol.equals( "dav" ) )
+        {
+            protocol = "http";
+        }
+
+        if ( protocol.equals( "davs" ) )
+        {
+            protocol = "https";
+        }
+
+        String redirectUrl = protocol + "://localhost:" + realServer.getConnectors()[0].getLocalPort();
+
+        RedirectHandler redirectHandler = new RedirectHandler( "Found", 303, redirectUrl, repositoryDirectory );
+
+        redirectServer.setHandler( redirectHandler );
+
+        redirectServer.start();
+
+        try
+        {
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+            Repository repository = new Repository( "foo", getRepositoryUrl( redirectServer ) );
+            wagon.connect( repository );
+
+            File sourceFile = new File( repositoryDirectory, "test-secured-put-resource" );
+            sourceFile.delete();
+            assertFalse( sourceFile.exists() );
+
+            File tempFile = File.createTempFile( "wagon", "tmp" );
+            tempFile.deleteOnExit();
+            String content = "put top secret";
+            FileUtils.fileWrite( tempFile.getAbsolutePath(), content );
+
+            try
+            {
+                wagon.put( tempFile, "test-secured-put-resource" );
+            }
+            finally
+            {
+                tempFile.delete();
+            }
+
+            assertEquals( content, FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+
+        }
+        finally
+        {
+            realServer.stop();
+            redirectServer.stop();
+        }
+    }
+
+
+    public void testRedirectPutFileRelativeUrl()
+        throws Exception
+    {
+        Server realServer = new Server( 0 );
+        addConnectors( realServer );
+        File repositoryDirectory = getRepositoryDirectory();
+        FileUtils.deleteDirectory( repositoryDirectory );
+        repositoryDirectory.mkdirs();
+
+        PutHandler putHandler = new PutHandler( repositoryDirectory );
+
+        realServer.setHandler( putHandler );
+
+        realServer.start();
+
+        Server redirectServer = new Server( 0 );
+
+        addConnectors( redirectServer );
+
+        RedirectHandler redirectHandler =
+            new RedirectHandler( "Found", 303, "/redirectRequest/foo", repositoryDirectory );
+
+        redirectServer.setHandler( redirectHandler );
+
+        redirectServer.start();
+
+        try
+        {
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+            Repository repository = new Repository( "foo", getRepositoryUrl( redirectServer ) );
+            wagon.connect( repository );
+
+            File sourceFile = new File( repositoryDirectory, "/redirectRequest/foo/test-secured-put-resource" );
+            sourceFile.delete();
+            assertFalse( sourceFile.exists() );
+
+            File tempFile = File.createTempFile( "wagon", "tmp" );
+            tempFile.deleteOnExit();
+            String content = "put top secret";
+            FileUtils.fileWrite( tempFile.getAbsolutePath(), content );
+
+            try
+            {
+                wagon.put( tempFile, "test-secured-put-resource" );
+            }
+            finally
+            {
+                tempFile.delete();
+            }
+
+            assertEquals( content, FileUtils.fileRead( sourceFile.getAbsolutePath() ) );
+
+        }
+        finally
+        {
+            realServer.stop();
+            redirectServer.stop();
+        }
+    }
+
+
+    public static class RedirectHandler
+        extends AbstractHandler
+    {
+        String reason;
+
+        int retCode;
+
+        String redirectUrl;
+
+        File repositoryDirectory;
+
+        public List<HandlerRequestResponse> handlerRequestResponses = new ArrayList<HandlerRequestResponse>();
+
+        RedirectHandler( String reason, int retCode, String redirectUrl, File repositoryDirectory )
+        {
+            this.reason = reason;
+            this.retCode = retCode;
+            this.redirectUrl = redirectUrl;
+            this.repositoryDirectory = repositoryDirectory;
+        }
+
+        public void handle( String s, HttpServletRequest req, HttpServletResponse resp, int i )
+            throws IOException, ServletException
+        {
+            if ( req.getRequestURI().contains( "redirectRequest" ) )
+            {
+                PutHandler putHandler = new PutHandler( this.repositoryDirectory );
+                putHandler.handle( s, req, resp, i );
+                handlerRequestResponses.add(
+                    new HandlerRequestResponse( req.getMethod(), ( (Response) resp ).getStatus(),
+                                                req.getRequestURI() ) );
+                return;
+            }
+            resp.setStatus( this.retCode );
+            resp.sendRedirect( this.redirectUrl + "/" + req.getRequestURI() );
+            handlerRequestResponses.add(
+                new HandlerRequestResponse( req.getMethod(), ( (Response) resp ).getStatus(), req.getRequestURI() ) );
+        }
+    }
+
+
     private void runTestProxiedRequest( ProxyInfo proxyInfo, TestHeaderHandler handler )
         throws Exception
     {
@@ -471,8 +950,6 @@ public abstract class HttpWagonTestCase
         {
             StreamingWagon wagon = (StreamingWagon) getWagon();
 
-            System.out.println( " wagon hashCode " + wagon.hashCode() );
-
             Repository testRepository = new Repository( "id", "http://www.example.com/" );
 
             String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
@@ -550,11 +1027,67 @@ public abstract class HttpWagonTestCase
         runTestSecuredGet( authInfo );
     }
 
+
     public void runTestSecuredGet( AuthenticationInfo authInfo )
         throws Exception
     {
         String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
         Server server = createSecurityServer( localRepositoryPath );
+
+        server.start();
+
+        try
+        {
+            StreamingWagon wagon = (StreamingWagon) getWagon();
+
+            Repository testRepository = new Repository( "id", getRepositoryUrl( server ) );
+
+            File sourceFile = new File( localRepositoryPath, "test-secured-resource" );
+            FileUtils.fileWrite( sourceFile.getAbsolutePath(), "top secret" );
+
+            wagon.connect( testRepository, authInfo );
+
+            File file = File.createTempFile( "wagon-test", "txt" );
+
+            try
+            {
+                wagon.get( "test-secured-resource", file );
+            }
+            finally
+            {
+                wagon.disconnect();
+            }
+
+            FileInputStream in = new FileInputStream( file );
+
+            assertEquals( "top secret", IOUtil.toString( in ) );
+
+            TestSecurityHandler securityHandler = (TestSecurityHandler) ( (Context) server.getHandler() ).getHandler();
+            testPreemptiveAuthenticationGet( securityHandler, supportPreemptiveAuthenticationGet() );
+
+        }
+        finally
+        {
+            server.stop();
+        }
+    }
+
+
+    public void testSecuredGetToStream()
+        throws Exception
+    {
+        AuthenticationInfo authInfo = new AuthenticationInfo();
+        authInfo.setUserName( "user" );
+        authInfo.setPassword( "secret" );
+        runTestSecuredGetToStream( authInfo );
+    }
+
+    public void runTestSecuredGetToStream( AuthenticationInfo authInfo )
+        throws Exception
+    {
+        String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
+        Server server = createSecurityServer( localRepositoryPath );
+
         server.start();
 
         try
@@ -579,6 +1112,9 @@ public abstract class HttpWagonTestCase
             }
 
             assertEquals( "top secret", out.toString() );
+
+            TestSecurityHandler securityHandler = (TestSecurityHandler) ( (Context) server.getHandler() ).getHandler();
+            testPreemptiveAuthenticationGet( securityHandler, supportPreemptiveAuthenticationGet() );
         }
         finally
         {
@@ -630,6 +1166,7 @@ public abstract class HttpWagonTestCase
     {
         String localRepositoryPath = FileTestUtils.getTestOutputDir().toString();
         Server server = createSecurityServer( localRepositoryPath );
+
         server.start();
 
         try
@@ -886,7 +1423,7 @@ public abstract class HttpWagonTestCase
             server.stop();
         }
         assertEquals( putNumber, putHandler.putCallNumber );
-        testPreemptiveAuthentication( sh );
+        testPreemptiveAuthenticationPut( sh, supportPreemptiveAuthenticationPut() );
     }
 
     public void testNonSecuredPutFromStream()
@@ -970,7 +1507,7 @@ public abstract class HttpWagonTestCase
         assertEquals( putNumber, putHandler.putCallNumber );
         if ( addSecurityHandler )
         {
-            testPreemptiveAuthentication( sh );
+            testPreemptiveAuthenticationPut( sh, supportPreemptiveAuthenticationPut() );
         }
 
         // ensure we didn't use chunked transfer which doesn't work on ngnix
@@ -984,28 +1521,37 @@ public abstract class HttpWagonTestCase
     }
 
 
-    protected abstract boolean supportPreemptiveAuthentication();
+    protected abstract boolean supportPreemptiveAuthenticationPut();
 
-    protected boolean supportProxyPreemptiveAuthentication()
+    protected abstract boolean supportPreemptiveAuthenticationGet();
+
+    protected abstract boolean supportProxyPreemptiveAuthentication();
+
+    protected void testPreemptiveAuthenticationGet( TestSecurityHandler sh, boolean preemptive )
     {
-        return false;
+        testPreemptiveAuthentication( sh, preemptive );
     }
 
-    protected void testPreemptiveAuthentication( TestSecurityHandler sh )
+    protected void testPreemptiveAuthenticationPut( TestSecurityHandler sh, boolean preemptive )
+    {
+        testPreemptiveAuthentication( sh, preemptive );
+    }
+
+    protected void testPreemptiveAuthentication( TestSecurityHandler sh, boolean preemptive )
     {
 
-        if ( supportPreemptiveAuthentication() )
+        if ( preemptive )
         {
-            assertEquals( "not 1 security handler use " + sh.securityHandlerRequestReponses, 1,
-                          sh.securityHandlerRequestReponses.size() );
-            assertEquals( 200, sh.securityHandlerRequestReponses.get( 0 ).responseCode );
+            assertEquals( "not 1 security handler use " + sh.handlerRequestResponses, 1,
+                          sh.handlerRequestResponses.size() );
+            assertEquals( 200, sh.handlerRequestResponses.get( 0 ).responseCode );
         }
         else
         {
-            assertEquals( "not 2 security handler use " + sh.securityHandlerRequestReponses, 2,
-                          sh.securityHandlerRequestReponses.size() );
-            assertEquals( 401, sh.securityHandlerRequestReponses.get( 0 ).responseCode );
-            assertEquals( 200, sh.securityHandlerRequestReponses.get( 1 ).responseCode );
+            assertEquals( "not 2 security handler use " + sh.handlerRequestResponses, 2,
+                          sh.handlerRequestResponses.size() );
+            assertEquals( 401, sh.handlerRequestResponses.get( 0 ).responseCode );
+            assertEquals( 200, sh.handlerRequestResponses.get( 1 ).responseCode );
 
         }
     }
@@ -1060,7 +1606,7 @@ public abstract class HttpWagonTestCase
         }
     }
 
-    static class PutHandler
+    public static class PutHandler
         extends AbstractHandler
     {
         private final File resourceBase;
@@ -1069,6 +1615,8 @@ public abstract class HttpWagonTestCase
 
         public int putCallNumber = 0;
 
+        public List<HandlerRequestResponse> handlerRequestResponses = new ArrayList<HandlerRequestResponse>();
+
         public PutHandler( File repositoryDirectory )
         {
             this.resourceBase = repositoryDirectory;
@@ -1100,7 +1648,6 @@ public abstract class HttpWagonTestCase
                 in.close();
                 out.close();
             }
-            System.out.println( "put file " + request.getPathInfo() );
             putCallNumber++;
             DeployedResource deployedResource = new DeployedResource();
 
@@ -1111,6 +1658,10 @@ public abstract class HttpWagonTestCase
             deployedResources.add( deployedResource );
 
             response.setStatus( HttpServletResponse.SC_CREATED );
+
+            handlerRequestResponses.add(
+                new HandlerRequestResponse( request.getMethod(), ( (Response) response ).getStatus(),
+                                            request.getRequestURI() ) );
         }
     }
 
@@ -1118,8 +1669,7 @@ public abstract class HttpWagonTestCase
         extends TestHeaderHandler
     {
 
-        List<SecurityHandlerRequestReponse> securityHandlerRequestReponses =
-            new ArrayList<SecurityHandlerRequestReponse>();
+        List<HandlerRequestResponse> handlerRequestResponses = new ArrayList<HandlerRequestResponse>();
 
         public void handle( String target, HttpServletRequest request, HttpServletResponse response, int dispatch )
             throws IOException, ServletException
@@ -1127,14 +1677,16 @@ public abstract class HttpWagonTestCase
             System.out.println( " handle proxy request" );
             if ( request.getHeader( "Proxy-Authorization" ) == null )
             {
-                securityHandlerRequestReponses.add( new SecurityHandlerRequestReponse( request.getMethod(), 407 ) );
+                handlerRequestResponses.add(
+                    new HandlerRequestResponse( request.getMethod(), 407, request.getRequestURI() ) );
                 response.setStatus( 407 );
                 response.addHeader( "Proxy-Authenticate", "Basic realm=\"Squid proxy-caching web server\"" );
 
                 ( (Request) request ).setHandled( true );
                 return;
             }
-            securityHandlerRequestReponses.add( new SecurityHandlerRequestReponse( request.getMethod(), 200 ) );
+            handlerRequestResponses.add(
+                new HandlerRequestResponse( request.getMethod(), 200, request.getRequestURI() ) );
             super.handle( target, request, response, dispatch );
         }
     }
@@ -1142,7 +1694,9 @@ public abstract class HttpWagonTestCase
     private static class TestHeaderHandler
         extends AbstractHandler
     {
-        public Map headers = Collections.EMPTY_MAP;
+        public Map<String, String> headers = Collections.emptyMap();
+
+        public List<HandlerRequestResponse> handlerRequestResponses = new ArrayList<HandlerRequestResponse>();
 
         public TestHeaderHandler()
         {
@@ -1160,7 +1714,11 @@ public abstract class HttpWagonTestCase
 
             response.setContentType( "text/plain" );
             response.setStatus( HttpServletResponse.SC_OK );
-            response.getWriter().println( "Hello, World!" );
+            response.getWriter().print( "Hello, World!" );
+
+            handlerRequestResponses.add(
+                new HandlerRequestResponse( request.getMethod(), ( (Response) response ).getStatus(),
+                                            request.getRequestURI() ) );
 
             ( (Request) request ).setHandled( true );
         }
@@ -1191,8 +1749,7 @@ public abstract class HttpWagonTestCase
         extends SecurityHandler
     {
 
-        public List<SecurityHandlerRequestReponse> securityHandlerRequestReponses =
-            new ArrayList<SecurityHandlerRequestReponse>();
+        public List<HandlerRequestResponse> handlerRequestResponses = new ArrayList<HandlerRequestResponse>();
 
         @Override
         public void handle( String target, HttpServletRequest request, HttpServletResponse response, int dispatch )
@@ -1200,33 +1757,36 @@ public abstract class HttpWagonTestCase
         {
             String method = request.getMethod();
             super.handle( target, request, response, dispatch );
-            System.out.println( "method in SecurityHandler: " + method );
 
-            securityHandlerRequestReponses.add(
-                new SecurityHandlerRequestReponse( method, ( (Response) response ).getStatus() ) );
+            handlerRequestResponses.add(
+                new HandlerRequestResponse( method, ( (Response) response ).getStatus(), request.getRequestURI() ) );
         }
 
     }
 
-    public static class SecurityHandlerRequestReponse
+    public static class HandlerRequestResponse
     {
         public String method;
 
         public int responseCode;
 
-        private SecurityHandlerRequestReponse( String method, int responseCode )
+        public String requestUri;
+
+        private HandlerRequestResponse( String method, int responseCode, String requestUri )
         {
             this.method = method;
             this.responseCode = responseCode;
+            this.requestUri = requestUri;
         }
 
         @Override
         public String toString()
         {
             final StringBuilder sb = new StringBuilder();
-            sb.append( "SecurityHandlerRequestReponse" );
+            sb.append( "HandlerRequestResponse" );
             sb.append( "{method='" ).append( method ).append( '\'' );
             sb.append( ", responseCode=" ).append( responseCode );
+            sb.append( ", requestUri='" ).append( requestUri ).append( '\'' );
             sb.append( '}' );
             return sb.toString();
         }
