@@ -19,6 +19,7 @@ package org.apache.maven.wagon.providers.http;
  * under the License.
  */
 
+import org.apache.commons.io.IOUtils;
 import org.apache.maven.wagon.ConnectionException;
 import org.apache.maven.wagon.InputData;
 import org.apache.maven.wagon.OutputData;
@@ -55,7 +56,7 @@ import java.util.zip.GZIPInputStream;
  * LightweightHttpWagon, using JDK's HttpURLConnection.
  *
  * @author <a href="michal.maczka@dimatics.com">Michal Maczka</a>
- * @version $Id: LightweightHttpWagon.java 1208429 2011-11-30 13:54:26Z olamy $
+ *
  * @plexus.component role="org.apache.maven.wagon.Wagon" role-hint="http" instantiation-strategy="per-lookup"
  * @see HttpURLConnection
  */
@@ -85,7 +86,7 @@ public class LightweightHttpWagon
     /**
      * @plexus.requirement
      */
-    private LightweightHttpWagonAuthenticator authenticator;
+    private volatile LightweightHttpWagonAuthenticator authenticator;
 
     /**
      * Builds a complete URL string from the repository URL and the relative path passed.
@@ -338,6 +339,7 @@ public class LightweightHttpWagon
     public void closeConnection()
         throws ConnectionException
     {
+        //FIXME WAGON-375 use persistent connection feature provided by the jdk
         if ( putConnection != null )
         {
             putConnection.disconnect();
@@ -365,13 +367,21 @@ public class LightweightHttpWagon
 
         InputStream is = inputData.getInputStream();
 
-        if ( is == null )
+        try
         {
-            throw new TransferFailedException(
-                url + " - Could not open input stream for resource: '" + resource + "'" );
-        }
 
-        return HtmlFileListParser.parseFileList( url, is );
+            if ( is == null )
+            {
+                throw new TransferFailedException(
+                    url + " - Could not open input stream for resource: '" + resource + "'" );
+            }
+
+            return HtmlFileListParser.parseFileList( url, is );
+        }
+        finally
+        {
+            IOUtils.closeQuietly( is );
+        }
     }
 
     public boolean resourceExists( String resourceName )
@@ -452,4 +462,14 @@ public class LightweightHttpWagon
     {
         this.preemptiveAuthentication = preemptiveAuthentication;
     }
+
+    public LightweightHttpWagonAuthenticator getAuthenticator()
+    {
+        return authenticator;
+    }
+
+    public void setAuthenticator( LightweightHttpWagonAuthenticator authenticator )
+    {
+        this.authenticator = authenticator;
+    }
 }
