@@ -31,6 +31,7 @@ import org.apache.commons.httpclient.NTCredentials;
 import org.apache.commons.httpclient.UsernamePasswordCredentials;
 import org.apache.commons.httpclient.auth.AuthScope;
 import org.apache.commons.httpclient.cookie.CookiePolicy;
+import org.apache.commons.httpclient.methods.EntityEnclosingMethod;
 import org.apache.commons.httpclient.methods.GetMethod;
 import org.apache.commons.httpclient.methods.HeadMethod;
 import org.apache.commons.httpclient.methods.PutMethod;
@@ -38,6 +39,8 @@ import org.apache.commons.httpclient.methods.RequestEntity;
 import org.apache.commons.httpclient.params.HttpMethodParams;
 import org.apache.commons.httpclient.util.DateParseException;
 import org.apache.commons.httpclient.util.DateUtil;
+import org.apache.commons.io.IOUtils;
+import org.apache.commons.lang.StringUtils;
 import org.apache.maven.wagon.InputData;
 import org.apache.maven.wagon.OutputData;
 import org.apache.maven.wagon.PathUtils;
@@ -50,16 +53,15 @@ import org.apache.maven.wagon.events.TransferEvent;
 import org.apache.maven.wagon.proxy.ProxyInfo;
 import org.apache.maven.wagon.repository.Repository;
 import org.apache.maven.wagon.resource.Resource;
-import org.codehaus.plexus.util.IOUtil;
-import org.codehaus.plexus.util.StringUtils;
 
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.URLEncoder;
+import java.nio.ByteBuffer;
 import java.text.SimpleDateFormat;
 import java.util.Date;
 import java.util.Iterator;
@@ -82,7 +84,9 @@ public abstract class AbstractHttpClientWagon
 
         private final Wagon wagon;
 
-        private final File source;
+        private File source;
+
+        private ByteBuffer byteBuffer;
 
         private RequestEntityImplementation( final InputStream stream, final Resource resource, final Wagon wagon,
                                              final File source )
@@ -94,23 +98,15 @@ public abstract class AbstractHttpClientWagon
             }
             else
             {
-                FileOutputStream fos = null;
                 try
                 {
-                    this.source = File.createTempFile( "http-wagon.", ".tmp" );
-                    this.source.deleteOnExit();
-
-                    fos = new FileOutputStream( this.source );
-                    IOUtil.copy( stream, fos );
+                    byte[] bytes = IOUtils.toByteArray( stream );
+                    this.byteBuffer = ByteBuffer.allocate( bytes.length );
+                    this.byteBuffer.put( bytes );
                 }
                 catch ( IOException e )
                 {
-                    fireTransferError( resource, e, TransferEvent.REQUEST_PUT );
-                    throw new TransferFailedException( "Failed to buffer stream contents to temp file for upload.", e );
-                }
-                finally
-                {
-                    IOUtil.close( fos );
+                    throw new TransferFailedException( e.getMessage(), e );
                 }
             }
 
@@ -142,10 +138,12 @@ public abstract class AbstractHttpClientWagon
                 new TransferEvent( wagon, resource, TransferEvent.TRANSFER_PROGRESS, TransferEvent.REQUEST_PUT );
             transferEvent.setTimestamp( System.currentTimeMillis() );
 
-            FileInputStream fin = null;
+            InputStream fin = null;
             try
             {
-                fin = new FileInputStream( source );
+                fin = this.source != null
+                    ? new FileInputStream( source )
+                    : new ByteArrayInputStream( this.byteBuffer.array() );
                 int remaining = Integer.MAX_VALUE;
                 while ( remaining > 0 )
                 {
@@ -165,7 +163,7 @@ public abstract class AbstractHttpClientWagon
             }
             finally
             {
-                IOUtil.close( fin );
+                IOUtils.closeQuietly( fin );
             }
 
             output.flush();
@@ -200,23 +198,41 @@ public abstract class AbstractHttpClientWagon
         // WAGON-273: default the cookie-policy to browser compatible
         client.getParams().setCookiePolicy( CookiePolicy.BROWSER_COMPATIBILITY );
 
+
+
         String username = null;
         String password = null;
+        String domain = null;
 
         if ( authenticationInfo != null )
         {
             username = authenticationInfo.getUserName();
 
+            if ( StringUtils.contains( username, "\\" ) )
+            {
+                String[] domainAndUsername = username.split( "\\\\" );
+                domain = domainAndUsername[0];
+                username = domainAndUsername[1];
+            }
+
             password = authenticationInfo.getPassword();
 
-            client.getParams().setAuthenticationPreemptive( true );
+
         }
 
         String host = getRepository().getHost();
 
         if ( StringUtils.isNotEmpty( username ) && StringUtils.isNotEmpty( password ) )
         {
-            Credentials creds = new UsernamePasswordCredentials( username, password );
+            Credentials creds;
+            if ( domain != null )
+            {
+                creds = new NTCredentials( username, password, host, domain );
+            }
+            else
+            {
+                creds = new UsernamePasswordCredentials( username, password );
+            }
 
             int port = getRepository().getPort() > -1 ? getRepository().getPort() : AuthScope.ANY_PORT;
 
@@ -306,16 +322,28 @@ public abstract class AbstractHttpClientWagon
     {
         StringBuilder url = new StringBuilder( getRepository().getUrl() );
         String[] parts = StringUtils.split( resource.getName(), "/" );
-        for ( String part : parts )// int i = 0; i < parts.length; i++ )
+        for ( String part : parts )
         {
             // TODO: Fix encoding...
-            // url += "/" + URLEncoder.encode( parts[i], System.getProperty("file.encoding") );
             if ( !url.toString().endsWith( "/" ) )
             {
                 url.append( '/' );
             }
             url.append( URLEncoder.encode( part ) );
         }
+        RequestEntityImplementation requestEntityImplementation =
+            new RequestEntityImplementation( stream, resource, this, source );
+        put( resource, source, requestEntityImplementation, url.toString() );
+
+    }
+
+    private void put( Resource resource, File source, RequestEntityImplementation requestEntityImplementation,
+                      String url )
+        throws TransferFailedException, AuthorizationException, ResourceDoesNotExistException
+    {
+
+        // preemptive true for put
+        client.getParams().setAuthenticationPreemptive( true );
 
         //Parent directories need to be created before posting
         try
@@ -327,18 +355,19 @@ public abstract class AbstractHttpClientWagon
             fireTransferError( resource, e, TransferEvent.REQUEST_GET );
         }
 
-        PutMethod putMethod = new PutMethod( url.toString() );
+        PutMethod putMethod = new PutMethod( url );
 
         firePutStarted( resource, source );
 
         try
         {
-            putMethod.setRequestEntity( new RequestEntityImplementation( stream, resource, this, source ) );
+            putMethod.setRequestEntity( requestEntityImplementation );
 
             int statusCode;
             try
             {
                 statusCode = execute( putMethod );
+
             }
             catch ( IOException e )
             {
@@ -359,6 +388,15 @@ public abstract class AbstractHttpClientWagon
                 case HttpStatus.SC_NO_CONTENT:  // 204
                     break;
 
+                // handle all redirect even if http specs says " the user agent MUST NOT automatically redirect the request unless it can be confirmed by the user"
+                case HttpStatus.SC_MOVED_PERMANENTLY: // 301
+                case HttpStatus.SC_MOVED_TEMPORARILY: // 302
+                case HttpStatus.SC_SEE_OTHER: // 303
+                    String relocatedUrl = calculateRelocatedUrl( putMethod );
+                    fireTransferDebug( "relocate to " + relocatedUrl );
+                    put( resource, source, requestEntityImplementation, relocatedUrl );
+                    return;
+
                 case SC_NULL:
                 {
                     TransferFailedException e = new TransferFailedException( "Failed to transfer file: " + url );
@@ -391,6 +429,14 @@ public abstract class AbstractHttpClientWagon
         }
     }
 
+    protected String calculateRelocatedUrl( EntityEnclosingMethod method )
+    {
+        Header locationHeader = method.getResponseHeader( "Location" );
+        String locationField = locationHeader.getValue();
+        // is it a relative Location or a full ?
+        return locationField.startsWith( "http" ) ? locationField : getURL( getRepository() ) + '/' + locationField;
+    }
+
     protected void mkdirs( String dirname )
         throws IOException
     {
@@ -407,6 +453,7 @@ public abstract class AbstractHttpClientWagon
         }
         url.append( resourceName );
         HeadMethod headMethod = new HeadMethod( url.toString() );
+
         int statusCode;
         try
         {
@@ -572,6 +619,7 @@ public abstract class AbstractHttpClientWagon
         url.append( resource.getName() );
 
         getMethod = new GetMethod( url.toString() );
+
         long timestamp = resource.getLastModified();
         if ( timestamp > 0 )
         {
@@ -722,7 +770,7 @@ public abstract class AbstractHttpClientWagon
     protected void putFromStream( InputStream stream, Resource resource )
         throws TransferFailedException, AuthorizationException, ResourceDoesNotExistException
     {
-        putFromStream( stream, resource.getName( ), -1, -1 );
+        putFromStream( stream, resource.getName(), -1, -1 );
     }
 
     @Override
